Attribute VB_Name = "basCommonDialog"
Option Explicit

Private Declare Function GetSaveFileName Lib "comdlg32.dll" Alias "GetSaveFileNameA" (pOpenFileName As OPENFileName) As Long
Private Declare Function GetOpenFileName Lib "comdlg32.dll" Alias "GetOpenFileNameA" (pOpenFileName As OPENFileName) As Long
  
Public Enum OfnFlags
    OFN_ALLOWMULTISELECT = &H200        'Allow multiple files to be selected
    OFN_CREATEPROMPT = &H2000           'Prompt to create files that don't exist
    OFN_DONTADDTORECENT = &H2000000     'Don't add file to Windows' Recent Files list
    OFN_ENABLESIZING = &H800000         'Allow user to resize dialog
    OFN_EXPLORER = &H80000              'User explorer style (only takes effect if OFN_ALLOWMULTISELECT is used
    OFN_FILEMUSTEXIST = &H1000          'User can only enter existing FileNames. Must also set OFN_PATHMUSTEXIST
    OFN_FORCESHOWHIDDEN = &H10000000    'Show hidden or system files regardless of Explorer settings (won't show hidden system files)
    OFN_HIDEREADONLY = &H4              'Hide the bReadOnly checkbox
    OFN_LONGNAMES = &H200000            'If using non-explorer style dialog, set this to use long FileNames
    OFN_NOCHANGEDIR = &H8               'Don't change default directory when dialog closes
    OFN_NODEREFERENCELINKS = &H100000   'Don't replace .lnk FileNames with the path & FileName they point to
    OFN_NOLONGNAMES = &H40000           'If using non-explorer style dialog, use short FileNames
    OFN_NONETWORKBUTTON = &H20000       'Disable the network button
    OFN_NOTESTFILECREATE = &H10000      'Do not create file before closing the dialog
    OFN_NOVALIDATE = &H100              'Allow invalid characters in FileName
    OFN_PATHMUSTEXIST = &H800           'Don't accept paths that don't exist
    OFN_bReadOnly = &H1                  'Set read-only checkbox by default
    OFN_SHAREAWARE = &H4000             'Ignore sharing violations
    OFN_SHOWHELP = &H10                 'Show Help button. Owner window must handle the help request
End Enum

'Dummy declarations force capitalization of enum items
#If False Then
    Public OFN_ALLOWMULTISELECT
    Public OFN_CREATEPROMPT
    Public OFN_DONTADDTORECENT
    Public OFN_ENABLESIZING
    Public OFN_EXPLORER
    Public OFN_FILEMUSTEXIST
    Public OFN_FORCESHOWHIDDEN
    Public OFN_HIDEREADONLY
    Public OFN_LONGNAMES
    Public OFN_NOCHANGEDIR
    Public OFN_NODEREFERENCELINKS
    Public OFN_NOLONGNAMES
    Public OFN_NONETWORKBUTTON
    Public OFN_NOTESTFILECREATE
    Public OFN_NOVALIDATE
    Public OFN_PATHMUSTEXIST
    Public OFN_bReadOnly
    Public OFN_SHAREAWARE
    Public OFN_SHOWHELP
#End If
  
Private Type OPENFileName
  lStructSize As Long
  hwndOwner As Long
  hInstance As Long
  lpstrFilter As String
  lpstrCustomFilter As String
  nMaxCustFilter As Long
  nFilterIndex As Long
  lpstrFile As String
  nMaxFile As Long
  lpstrFileTitle As String
  nMaxFileTitle As Long
  lpstrInitialDir As String
  lpstrTitle As String
  nFlags As Long
  nFileOffset As Integer
  nFileExtension As Integer
  lpstrDefExt As String
  lCustData As Long
  lpfnHook As Long
  lpTemplateName As String
End Type
 
'FileOpenDialog and FileSaveDialog have the same set of arguments and return value, except bReadOnly is
'not used for FileSaveDialog.
'
'sFilter     File extension filter made of pairs of descriptions and filespecs separated by pipes.
'            Multiple filespecs may be separated by semicolons. For example:
'            "All Files (*.*)|*.*|Graphic Files|*.jpg;*.png|"
'            nFlags      nFlags from OfnFlags (default=0)
'            bReadOnly   Set to TRUE by FileOpenDialog if read-only was selected by the user
'
'Returns path & filename selected or VbNullString if user cancelled or there was an error. If the user doesn't
'enter an extension, one is appended based on the selected filter. If the selected filter has multiple extensions
'the last one will be used.
'
' Notes: Sometimes VB gives a compiler error saying you need a constant if you pass a OFN flag to the function.
'        Creating the exe file seems to fix this for running the uncompiled code also
'
Function FileOpenDialog(Optional ByVal sTitle As String, Optional ByVal sFolder As String, _
                        Optional ByVal sFilter As String = "All Files|*.*|", Optional ByVal nFlags As OfnFlags, Optional ByRef bReadOnly As Boolean)
    FileOpenDialog = FileDialog(sTitle, sFolder, sFilter, nFlags, bReadOnly, bSaveMode:=False)
End Function

Function FileSaveDialog(Optional ByVal sTitle As String, Optional ByVal sFolder As String, _
                        Optional ByVal sFilter As String = "All Files|*.*|", Optional ByVal nFlags As OfnFlags)
    Dim bUnused As Boolean
    
    FileSaveDialog = FileDialog(sTitle, sFolder, sFilter, nFlags, bUnused, bSaveMode:=True)
End Function

'This function handles both open and save dialogs
Private Function FileDialog(ByVal sTitle As String, ByVal sFolder As String, ByVal sFilter As String, ByVal nFlags As OfnFlags, ByRef bReadOnly, ByVal bSaveMode As Boolean)
    Dim OpenFile As OPENFileName
    Dim nResult As Long
    Dim sFileName As String
    Dim sExtension As String
    
    If Right$(sFilter, 1) <> "|" Then    'Add 2 terminating pipes if there are none
        sFilter = sFilter & "||"
    ElseIf Right$(sFilter, 2) <> "|" Then    'Add 1 terminating pipe if there is only 1
        sFilter = sFilter & "|"
    End If

    sFilter = Replace(sFilter, "|", vbNullChar)
 
    With OpenFile
        .lStructSize = Len(OpenFile)
        .lpstrFilter = sFilter
        .nFilterIndex = 0
        .lpstrFile = sFolder & String$(257 - Len(sFolder), 0)
        .nMaxFile = Len(.lpstrFile) - 1
        .lpstrFileTitle = .lpstrFile
        .nMaxFileTitle = .nMaxFile
        .lpstrInitialDir = sFolder
        .lpstrTitle = sTitle
        .nFlags = nFlags
   
        If bSaveMode = True Then
            nResult = GetSaveFileName(OpenFile)
        Else
            nResult = GetOpenFileName(OpenFile)
        End If
        
        If nResult = 0 Then
           sFileName = vbNullString
        Else
            sFileName = TrimNull(.lpstrFile)
            If FileExtension(sFileName) = vbNullString Then
                sExtension = GetSelectedExtension(.lpstrFilter, .nFilterIndex)
                If sExtension <> "*" Then
                    sFileName = sFileName & "." & sExtension
                End If
            End If
            
            bReadOnly = ((.nFlags And OFN_bReadOnly) = OFN_bReadOnly)
        End If
        
    End With
    
    FileDialog = sFileName
End Function

Private Function GetSelectedExtension(ByVal sFilters As String, ByVal nFilterIndex As Long) As String
    Dim sFilterArray() As String
    sFilterArray = Split(sFilters, vbNullChar)
    GetSelectedExtension = FileExtension(sFilterArray(nFilterIndex * 2 - 1))
End Function

Private Function TrimNull(ByVal sText As String) As String
    TrimNull = Left(sText, InStr(sText, vbNullChar) - 1)
End Function

'Return the file extension from a path & filename
Private Function FileExtension(ByVal sFile As String) As String
    Dim nDotPosition As Long
    
    sFile = Filename(sFile)
    nDotPosition = InStrRev(sFile, ".")
    
    If nDotPosition > 1 Then
        FileExtension = Right$(sFile, Len(sFile) - nDotPosition)
    Else
        FileExtension = vbNullString
    End If
    
End Function

'Return the file name from a path & filename
Private Function Filename(ByVal PathName As String) As String
    Dim Position As Long
    
    Position = InStrRev(PathName, "\", Compare:=vbTextCompare)
    Filename = Right$(PathName, Len(PathName) - Position)
End Function


